(*
        SpaceSweeper helper 
        Writes level along with resources into game.dat 
        Used by MakeLevel.pas 

        Author: Piotr Skowronek, pskowronek@adfinem.net
        Written in: 1997
        Licence: LGPL
 *)

Unit WRITEDAT;      { Dostosowany do GRY - Space Sweeper }
{$I+,R-,G+,S-,D+,Q-,V-,B-}
interface          { DAT writer }
uses DOS,STRINGS,TYPES,MISC;


(**** Zapis DATw ***)
procedure InitResourceFile(name: string);      { Tworzy nowy z nagwkiem }
procedure AddKit(name: string; pos: longint);  { Dodaje nastpny KIT }
function  WriteBranches(name: string; Kit: T_kit): longint;
function  NewArmed(score: word; x1,y1,x2,y2: word; FireAim: boolean; what,next: pointer): pointer;
function  NewBranch(score: word; what,next: pointer): pointer;
function  NewShip(name: string;ReadPal: Boolean; next: pointer): pointer;

procedure WriteEpisodes(name: string; Epi: P_Episodes);

implementation


(****************************** Odczyt BMP'w *******************************)
procedure Read_Sprite(path: string; var BMP_Sprite: T_BMP_Sprite; ReadPal: Boolean);
Type
    tab = array [1..256] of record
                                  r,g,b,nic : byte;
                            end;
Var i          : word;
    bmp        : file;
    BMPFileHdr : ^BMP_File_Hdr_Type;
    BMPInfoHdr : ^BMP_Info_Hdr_Type;
    Tym        : ^tab;
    Head       : dword;
    Xmax,Ymax  : integer;
    P          : pointer;
begin
FileMode:=0;
assign(bmp,path);
reset(bmp,1);
Check4Error(path);

BMPFileHdr:=Reserve_Mem(SizeOf(BMP_File_Hdr_Type));
BMPInfoHdr:=Reserve_Mem(SizeOf(BMP_Info_Hdr_Type));
BlockRead(bmp,BMPFileHdr^,SizeOf(BMP_File_Hdr_Type));
BlockRead(bmp,BMPInfoHdr^,SizeOf(BMP_Info_Hdr_Type));
IF (BMPFileHdr^.bfType<>256*77+66) or (BMPInfoHdr^.biPlanes<>1) or
   (BMPInfoHdr^.biBitCount<>8) or (BMPInfoHdr^.biCompression<>0) Then
   WriteExitMsg('FATAL: Unsupported BMP file.'+#10+#13+
                '       File name: ['+path+'].');
Xmax:=BMPinfoHdr^.biWidth;
Ymax:=BMPInfoHdr^.biHeight;
Head:=BMPFileHdr^.bfOffBits;
If xmax mod 4<>0 then xmax:=(trunc(xmax/4))*4+4;
FreeMem(BMPFileHdr,SizeOf(BMP_File_Hdr_Type));
FreeMem(BMPInfoHdr,SizeOf(BMP_Info_Hdr_Type));

IF ReadPal Then
begin
Tym:=Reserve_Mem(SizeOf(tab));
BMP_Sprite.Pal:=Reserve_Mem(SizeOf(paleta));
seek(bmp,$36);
BlockRead(bmp,Tym^,sizeof(Tab));               { Get Palette from BMP }
for i:=1 to 256 do
    begin
         BMP_Sprite.Pal^[i].R:=Tym^[i].B shr 2;
         BMP_Sprite.Pal^[i].G:=Tym^[i].G shr 2;
         BMP_Sprite.Pal^[i].B:=Tym^[i].R shr 2;
    end;
FreeMem(Tym,SizeOf(tab));
BMP_Sprite.Pal_Exist:=True;
end else
BMP_Sprite.Pal_Exist:=False;


BMP_Sprite.Data:=Reserve_Mem(FileSize(BMP)-head);

Seek(BMP,Head);
BlockRead(BMP,BMP_Sprite.Data^,FileSize(BMP)-Head);
BMP_Sprite.Xmax:=Xmax;
BMP_Sprite.Ymax:=Ymax;
BMP_Sprite.Size:=FileSize(BMP)-head;

close(bmp);
end;


function NewShip(name: string;ReadPal: Boolean; next: pointer): pointer;
var  p: P_Image;
begin
     New(p);
     Write('Reading: ',name:12);
     Read_Sprite(name, p^.img, ReadPal);
     Writeln(' -> ',p^.img.xmax,'x',p^.img.ymax);
     p^.next:=next;
     NewShip:=p;
end;


function NewBranch(score: word; what,next: pointer): pointer;
var  p: P_Images;
begin
     New(p);
     p^.score:=score;
     p^.image:=what;
     p^.next:=next;
     NewBranch:=p;
end;


function NewArmed(score: word; x1,y1,x2,y2: word; FireAim: boolean; what,next: pointer): pointer;
var  p: P_Images;
begin
     New(p);
     p^.score:=score;
     p^.Fire.x1:=x1;
     p^.Fire.y1:=y1;
     p^.Fire.x2:=x2;
     p^.Fire.y2:=y2;
     p^.Fire.Able:=Yes;
     p^.Fire.Aim:=FireAim;
     p^.Fire.Power:=1;
     p^.image:=what;
     p^.next:=next;
     NewArmed:=p;
end;


procedure InitResourceFile(name: string);  {  Utworzenie pliku z nagwkiem }
var  f     : file;
     t_res : T_resource;
begin
     Assign(f,name);
     rewrite(f,1);
     StrCopy(@T_res.Head.ID,ID);
     T_Res.ImageKit:=nil;
     T_Res.Episodes:=nil;
     Blockwrite(f,T_res,sizeof(t_res));
     close(f);
end;

procedure AddKit(name: string; pos: longint);
var  head : T_resource;
     kit  : T_kit;
     f    : file;
begin
     Assign(f,name);
     reset(f,1);
     blockread(f,head,sizeof(head));
     seek(f,longint(head.imagekit));
     if head.imagekit=nil then
        begin
             seek(f,0);
             head.imagekit:=pointer(pos);
             blockwrite(f,head,sizeof(head));
        end else
     repeat
           blockread(f,kit,sizeof(kit));
           Check4Error(name);
           if kit.next=nil Then
              begin
                   kit.next:=pointer(pos);
                   seek(f,filepos(f)-sizeof(kit));
                   blockwrite(f,kit,sizeof(kit));
                   break;
              end;
           seek(f,longint(kit.next));
     until false;
     close(f);
end;


function WriteBranches(name: string; Kit: T_Kit): longint;

function NoOfBranches(p: p_images): word;
var  count : word;
begin
     count:=1;
     repeat
          If p^.next=nil Then break
                         Else begin p:=p^.next; inc(count); end;
     until false;
     NoOfBranches:=count;
end;

procedure write_tab(p: p_images; var F: file);
type t_tmp  = record
                    fpos      : longint;
                    p_branche : p_images;
                    t_branche : t_images;
                    p_sub     : p_image;
                    t_sub     : t_image;
                    p_bmp     : p_bmp_sprite;
                    t_bmp     : t_bmp_sprite;
              end;
const datbeg : pchar = 'DATA ->';
      datend : pchar = '<- DATA';
      palbeg : pchar = 'PAL ->';
      palend : pchar = '<- PAL';
var
     i,j     : word;
     temp1   : t_tmp;
     temp2   : t_tmp;
     temp3   : t_tmp;
     tympos1 : longint;
     tympos2 : longint;
     p2      : p_image;

begin
     repeat
          temp1.fpos:=FilePos(F);          { Rozdzielenie }
          temp1.t_branche:=p^;
          temp1.p_branche:=p;

          p2:=p^.image;
          seek(f,FilePos(f)+SizeOf(temp1.t_branche));

          repeat                            { Kolejne Bitmapy }
                   temp2.t_sub:=p2^;
                   temp2.p_sub:=p2;
                   temp2.fpos:=FilePos(f);
                   seek(f,FilePos(f)+SizeOf(temp2.t_sub));


                   if temp2.t_sub.img.pal_exist Then
                   begin                    { Write PAL if exists }
                   write('PAL -> ');
                   blockwrite(f,palbeg^,StrLen(palbeg)+1);   { PAL -> }
                   temp2.t_sub.img.pal:=pointer(filepos(f)); { Where in file ? }
                   blockwrite(f,temp2.p_sub^.img.pal^,3*256);{ Write PAL }
                   blockwrite(f,palend^,StrLen(palend)+1);   { <- PAL }
                   end;
                   blockwrite(f,datbeg^,StrLen(datbeg)+1);   { DATA -> }
                   temp2.t_sub.img.data:=pointer(Filepos(f));{ Where in file ? }

                                            { Ciaa Bitmap }
                   blockwrite(f,temp2.p_sub^.img.data^,temp2.t_sub.img.size);
                   blockwrite(f,datend^,StrLen(datend)+1);
                   Write(temp2.t_sub.img.size,', ');

                   tympos2:=FilePos(f);
                   If p2^.next<>nil Then temp2.t_sub.next:=pointer(FilePos(f))
                                    Else temp2.t_sub.next:=nil;
                   seek(f,temp2.fpos);
                   blockwrite(f,temp2.t_sub,sizeof(temp2.t_sub));
                   seek(f,tympos2);
                   If p2^.next=nil then break;
                   p2:=p2^.next;
          until false;
          Writeln(#8,#8,' -> END.');

          p:=p^.next;
          tympos1:=FilePos(f);              { Zapisanie pos w pliku do NEXT }
          if p<>nil Then temp1.t_branche.next:=pointer(tympos1)
                    else temp1.t_branche.next:=nil;
          temp1.t_branche.image:=pointer(temp1.fpos+Sizeof(temp1.t_branche));
          seek(f,temp1.fpos);
          blockwrite(f,temp1.t_branche,sizeof(temp1.t_branche));
          seek(f,tympos1);

          if p=nil then break;
     Until false;
     Writeln('File size: ',FileSize(f));
end;

var   f       : file;
      tpos    : longint;
      Tkit    : T_Kit;
begin
     filemode:=2;
     assign(f,name);
     reset(f,1);
     check4error(name);

     tpos:=FileSize(f);
     Seek(f,FileSize(f)+Sizeof(Tkit));

     Writeln('Alian''s branches: ',NoOfBranches(Kit.Alians));
     Writeln('Explode''s branches: ',NoOfBranches(Kit.Explodes));
     Writeln('Fight''s branches: ',NoOfBranches(Kit.Fighters));

     Tkit.Alians:=pointer(filepos(f));
     write_tab(Kit.Alians,F);
     Tkit.Explodes:=pointer(filesize(f));
     write_tab(Kit.Explodes,F);
     Tkit.Fighters:=pointer(filesize(f));
     write_tab(Kit.Fighters,F);
     Tkit.Cockpit:=pointer(filesize(f));
     write_tab(Kit.Cockpit,F);
     Tkit.Scene:=pointer(filesize(f));
     write_tab(Kit.Scene,F);
     Tkit.Colors:=pointer(filesize(f));
     write_tab(Kit.Colors,F);
     Tkit.Next:=nil;

     seek(f,tpos);
     blockwrite(f,tkit,SizeOf(Tkit));
     close(f);
     WriteBranches:=tpos;
end;



(****************** Write Episodes *******************)
procedure WriteEpisodes(name: string; Epi: P_Episodes);
function WriteIntro(var f: file; P: P_intro): pointer;
const datbeg : pchar = 'INTRO >>';
      datend : pchar = '<< INTRO';

var  T_Int   : T_Intro;
     P_Int   : P_Intro;
     T_Img   : T_Image;
     P_Img   : P_Image;
     TymPos1 : longint;
     TymPos2 : longint;

begin
     if P = nil then exit;

     P_Int:=P;                    { Intro }
     seek(f,filesize(f));
     blockwrite(f,datbeg^,StrLen(datbeg)+1);  { Begin of INTRO info }
     WriteIntro:=Pointer(filepos(f));
     TymPos2:=FilePos(f);
     Seek(f,filepos(F)+Sizeof(T_Int));

     T_Int:=P_Int^;
     T_Int.Text:=Pointer(FilePos(f));       { Text }
     blockwrite(f,P_Int^.Text^,P_Int^.Size);
     P_Img:=T_Int.Image;                    { Image }
     T_Int.Image:=Pointer(FilePos(f));
     T_Img:=P_Img^;
     T_Img.Next:=nil;                     { Zalepianie (moe kiedy nie ?) }
     TymPos1:=FilePos(f);
     seek(f,FilePos(f)+sizeof(T_img));

     T_Img.img.Pal:=Pointer(Filepos(f));
     if T_Img.img.Pal_Exist Then
     BlockWrite(f,P_Img^.img.Pal^,3*256);
     T_Img.img.Data:=Pointer(Filepos(f));
     BlockWrite(f,P_Img^.img.Data^,P_Img^.img.Size);  { ciao BitMapy }

     blockwrite(f,datend^,StrLen(datend)+1);  { End of INTRO info }

     Seek(f,TymPos1);
     blockwrite(f,T_Img,Sizeof(T_img));     { Image }

     seek(f,TymPos2);                       { Intro }
     blockwrite(f,T_Int,SizeOf(T_int));

end;

const ship   : pchar = '> Ships pos >';

var  f       : file;
     head    : T_Resource;
     T_Epi   : T_Episodes;
     P_Epi   : P_Episodes;
     T_Shi   : T_ShipsPos;
     P_Shi   : P_ShipsPos;
     T_Lev   : T_Levels;
     P_Lev   : P_Levels;
     tym1    : longint;
     tym2    : longint;
     Nextpos : longint;
     TymPos1 : longint;
     TymPos2 : longint;

begin
     filemode:=2;
     assign(f,name);
     reset(f,1);
     check4error(name);
     blockread(f,head,sizeof(head));
     check4error(name);
     if head.episodes<> nil Then WriteExitMsg('Resource already have an episodes.');
     head.episodes:=pointer(filesize(f));
     seek(f,0);
     blockwrite(f,head,sizeof(head));

     P_Epi:=Epi;
     seek(f,filesize(f));
{     txt_mode;}
repeat
     T_Epi:=P_Epi^;
     tym1:=filepos(f);                      { Episode }
     Seek(f,filepos(f)+sizeof(T_Episodes));

     P_Lev:=T_Epi.Levels;
     T_Epi.Levels:=Pointer(filepos(f));
     writeln('epi');
  repeat
     writeln('levi');
     if P_Lev=nil then WriteExitMsg('Level pointer is null');
     T_Lev:=P_Lev^;
     tym2:=filepos(f);                      { Level }
     Seek(f,filepos(f)+sizeof(T_Levels));

     P_Shi:=T_Lev.ShipsPos;
     T_Lev.ShipsPos:=pointer(FilePos(f){+StrLen(ship)+1});
     repeat
            writeln('shipi');
{            blockwrite(f,ship^,StrLen(ship)+1);}
            T_Shi:=P_Shi^;
            if T_Shi.Next<>nil then T_Shi.Next:=pointer(filepos(f)+sizeof(t_shi))
                               else T_Shi.Next:=nil;
            blockwrite(f,T_Shi,SizeOf(T_Shi));
            if P_Shi^.Next = nil Then break;
            P_Shi:=P_Shi^.Next;
     until false;

     T_Lev.Intro:=WriteIntro(f,T_Lev.Intro);
     if P_Lev^.Next<>nil then T_Lev.Next:=Pointer(FileSize(f))   { Next }
                         else T_Lev.Next:=nil; { TEST 1 }

     NextPos:=FilePos(f);                     { test 2 }
     seek(f,tym2);                          { Level }
     blockwrite(f,T_Lev,sizeof(T_Lev));
     check4error(name);
     seek(f,NextPos);
     P_Lev:=P_Lev^.Next;
     if P_Lev = nil then break;
  until false;


     T_Epi.Intro:=WriteIntro(f,T_Epi.Intro);


     if P_Epi^.Next<>nil then T_Epi.Next:=pointer(FileSize(f));

     NextPos:=FilePos(f);
     seek(f,tym1);                          { Episode }
     blockwrite(f,T_Epi,sizeof(T_Episodes));
     check4error(name);
     P_Epi:=P_Epi^.Next;
     seek(f,NextPos);
     if P_Epi=nil Then break;
until false;
     check4error(name);
     close(f);
     writeln('halti');
{     halt;}
end;



end.